/* 2darray.h -- Allocate and destroy 2d arrays
 *
 * Author: Dylan Muir
 * Date: 7th November, 1999
 * Modified: 16th August, 2000
 * Version: 0.03
 */

#ifndef	__2darray_h
#define	__2darray_h

#include "stddefs.h"
#include <stdlib.h>

/* -- Function Allocate2DArray
 * Pre: 'array' is a pointer to a pointer to a pointer to char
 *      'xSize' > 0
 *      'ySize' > 0
 *      'elementSize' is the size of each element to allocate
 * Post: ('array' was allocated space for a 2d array of dimensions [xSize x ySize]
 *       && TRUE was returned) ||
 *       (there was a problem with allocation && FALSE was returned)
 * Note: NO initialisation is done!  The content of the array after allocation
 *       is undefined.
 * Note: the array is indexed 'array'[col][row]
 */
bool Allocate2DArray(char ***array, int xSize, int ySize, size_t elementSize);

/* -- Function Deallocate2DArray
 * Pre: 'array' is a pointer to a valid 2D array allocated by Allocate2DArray,
 *       the contents of which have already been destroyed
 *      'ySize' is the size of the first index in 'array'
 * Post: 'array' was deallocated
 * Note: This function does not destroy the contents of the array, as it has no
 *       idea what the contents are.  You must destroy the contents yourself.
 */
void Deallocate2DArray(char ***array, int xSize);

#endif	__2darray_h

/* --- END of 2darray.h --- */

