#include <math.h>
#include "vector_utils.h"
#include "cluster.h"
#include "avq_subs.h"

#define	MAX_CLUSTERS	400
#define	AVQ_TERM_LEVEL	0.001

#if defined(ibmpc) || defined(sparc)
#define EXP(m) exp(m)
#else
#define EXP(m) \
        (exp_array[((int) ((m) * exp_mult)) + exp_add])
#endif

#define	ABS(x)	((x) < 0 ? (0.0 - (x)) : (x))


extern	int	nn;		/* number of nodes */
extern	int	nt;		/* nn + ni + 1 size of activation arrays */
extern	int	np;		/* start of real nodes activations */
extern	int	ni;		/* number of inputs */
extern	int	nhc;		/* number of hidden/context units */

struct	cf {
	int	con;		/* connection flag */
	int	fix;		/* fixed flag */
	int	num;		/* group number */
	int	lim;		/* weight limits */
	float	min;		/* weight minimum */
	float	max;		/* weight maximum */
};

struct	nf {
	int	func;		/* activation function */
	int	dela;		/* delay flag */
	int	targ;		/* target flag */
};

extern	struct	cf	**cinfo;
extern	struct	nf	*ninfo;

extern	int	*clustnodes;		/* (nn+1) nodes for avq */
extern	int	*contxtnodes;		/* (nn+1) context nodes */
extern	int	loadflag;		/* flag to say we're loading initial state */

extern	long	epochlength;		/* number of sweeps per epoch */
extern	long	epochsforavq;		/* number of epochs each avq sweep */

extern	int	codebook_size;		/* size of codebook (number of clusters) */
extern	vector	**avq_codebook;		/* current codebook */
extern	vector	**patterns;		/* stored patterns */
extern	long	patterns_index;		/* index into patterns array */
extern	long	num_patterns;		/* number of patterns stored */

extern	double	avq_phi;		/* avq phi parameter */
extern	double	avq_t;			/* avq T parameter */

extern	float	err;			/* cumulative error */
extern	float	ce_err;			/* cumulative cross entropy error */
extern	float	avq_err;		/* error for avq modification */

int	*clusterMembership;		/* array of what vectors belong to what clusters */


AVQ_ResetVectors()
{
	AVQ_delete_patterns();
	patterns_index = 0;
}


AVQ_classify_hu_to_context(aold, amem, anew, awt, local, atarget)
	float	*aold;
	float	*amem;
	float	*anew;
	float	**awt;
	int	*local;
	float	*atarget;
{
	vector	*cluster_act;
	int	node,
		dim_index;
	double	alpha;
	float	new_act;

	cluster_act = avq_codebook[FindNearestNeighbour(patterns[patterns_index - 1], avq_codebook, codebook_size)];

	alpha = exp((0.0 - avq_phi) * avq_err);

	dim_index = 0;
	for (node = 1; node <= nn; node++) {
		if (contxtnodes[node]) {
			new_act = (float) ((1.0 - alpha) * patterns[patterns_index - 1] -> components[dim_index]);
			new_act += alpha * (double) (cluster_act -> components[dim_index]);
/*			new_act = 1.0 / (1.0 + EXP(0.0 - new_act)); */
			aold[node + ni] = anew[node + ni] = new_act;
			dim_index++;
			if (dim_index == nhc) break;
		}
	}
}

AVQ_ReClassify(void)
{
	double	lastError, thisError,
		avDist, closestDist,
		closest2Dist;
	int	slot,
		closest, oldClosest,
		closest2;
	bool	terminate;
	cluster	**clusters;

	for (slot = 1; slot < num_patterns; slot++)
		clusterMembership[slot] = 0;
	clusterMembership[0] = 1;

	avq_codebook = CodebookDestroy(avq_codebook, codebook_size);
	avq_codebook = InitialiseCodebook(MAX_CLUSTERS, patterns, num_patterns);
	clusters = MakeClusters(MAX_CLUSTERS);
	clusters[0] = ClusterFromCodebook(patterns, num_patterns);
	clusters[0] = ClusterRemoveVector(clusters[0], patterns[0]);
	codebook_size = 2;
	lastError = 999.0;

#ifdef	DEBUG
	fprintf(stderr, "AVQ:(%4.2f)[", avq_t);
	fflush(stdout);
#endif

	terminate = FALSE;
	while (!terminate) {
		slot = 0;
		while (slot < num_patterns) {
			avDist = CodebookAverageDistance(patterns[slot], avq_codebook, codebook_size);
			closest = FindNearestNeighbour(patterns[slot], avq_codebook, codebook_size);
			closestDist = VectorEuclideanDist(patterns[slot], avq_codebook[closest]);
/*			closest2 = FindNearestOtherCluster(avq_codebook, codebook_size, closest);
			closest2Dist = VectorEuclideanDist(avq_codebook[closest], avq_codebook[closest2]); */

			if (closestDist > (avDist * avq_t)) {
/*			if (ABS(closestDist - avDist) < (avDist * avq_t)) { */
/*			if (closestDist > (avq_t * closest2Dist)) { */
				closest = FreeCluster(avq_codebook, MAX_CLUSTERS);

				if (closest == -1) {
					perror("ERROR: Exceeded compiler limit on clusters");
					exit(1);
				}
				if (closest >= codebook_size)
					codebook_size = closest + 1;
			}

			oldClosest = clusterMembership[slot];
			clusters[oldClosest] = ClusterRemoveVector(clusters[oldClosest], patterns[slot]);
			if (avq_codebook[oldClosest] != NULL)
				VectorDeallocate(avq_codebook[oldClosest]);
			avq_codebook[oldClosest] = ClusterCentroid(clusters[oldClosest]);

			clusters[closest] = ClusterAddVector(clusters[closest], VectorMakeCopy(patterns[slot]));
			clusterMembership[slot] = closest;
			if (avq_codebook[closest] != NULL)
				VectorDeallocate(avq_codebook[closest]);
			avq_codebook[closest] = ClusterCentroid(clusters[closest]);

			slot++;
		}

		thisError = GetError(clusters, avq_codebook, codebook_size);
		if ((codebook_size >= 20) && (lastError >= thisError) && (lastError - thisError < AVQ_TERM_LEVEL))
			terminate = TRUE;
		lastError = thisError;
		if (codebook_size < 20) {
			avq_t -= 0.01;
#ifdef	DEBUG
			fprintf(stderr, "-");
#endif
		} else if (codebook_size > 80) {
			avq_t += 0.01;
#ifdef	DEBUG
			fprintf(stderr, "+");
#endif
		} else {
#ifdef	DEBUG
			fprintf(stderr, ".");
#endif
		}
#ifdef	DEBUG
		fflush(stdout);
#endif
	}

#ifdef	DEBUG
	fprintf(stderr, "] RMS error: %f (%d clusters)\n", thisError, codebook_size);
	fflush(stdout);
#endif

	clusters = DestroyClusters(clusters, MAX_CLUSTERS);
}


AVQ_check_hidden_context(void)
{
	int	node;
	int	count_clust, count_contxt;

	count_clust = 0;
	for (node = 1; node <= nn; node++)
		if (clustnodes[node]) count_clust++;

	count_contxt = 0;
	for (node = 1; node <= nn; node++)
		if (contxtnodes[node]) count_contxt++;

	if (count_contxt != count_clust) {
		perror("ERROR: number of context nodes must equal the number of cluster nodes");
		exit(1);
	}

	nhc = count_contxt;
}

AVQ_initialise(void)
{
	int	dIndex;

	if (nhc == 0) {
		perror("ERROR: num context/cluster units is zero!");
		exit(1);
	}

	if (!(patterns = (codebook) malloc(num_patterns * sizeof(vector *)))) {
		perror("ERRPR: could not allocate patterns array\n");
		exit(1);
	}
	AVQ_wipe_patterns();
	patterns_index = 0;

	if (!loadflag) {
		if (!(avq_codebook = CodebookNew(MAX_CLUSTERS))) {
			perror("ERROR: could not allocate initial codebook");
			exit(1);
		}

		avq_codebook[0] = VectorAllocate(nhc);	/* initial cluster is zero */
		for (dIndex = 0; dIndex < nhc; dIndex++)
			avq_codebook[0] -> components[dIndex] = 0.5;

		codebook_size = 1;
	}

	if (!(clusterMembership = (int *) malloc(num_patterns * sizeof(int)))) {
		perror("ERROR: could not allocate cluster membership array");
		exit(1);
	}
}

AVQ_add_pattern(aold)
	float	*aold;
{
	vector	*hu_activation;
	int	node,
		dimension_index;

	if (!(hu_activation = VectorAllocate(nhc))) {
		perror("ERROR: unable to allocate pattern");
		exit(1);
	}

	dimension_index = 0;
	for (node = 1; node <= nn; node++) {
		if (clustnodes[node]) {
			hu_activation -> components[dimension_index] = aold[node + ni];
			dimension_index++;
			if (dimension_index == nhc) break;
		}
	}

	patterns[patterns_index] = hu_activation;
	patterns_index++;
}


AVQ_wipe_patterns(void)
{
	int	pattern_index;

	for (pattern_index = 0; pattern_index < num_patterns; pattern_index++)
		patterns[pattern_index] = NULL;
}

AVQ_delete_patterns(void)
{
	int	pattern_index;

	for (pattern_index = 0; pattern_index < num_patterns; pattern_index++)
		patterns[pattern_index] = VectorDeallocate(patterns[pattern_index]);
}


/* --- END of avq.c --- */
