#include <math.h>
#include <stdio.h>
#ifdef ibmpc
#include <fcntl.h>
#include <sys/types.h>
#else
#include <fcntl.h>
#include <sys/file.h>
#include <sys/types.h>
#endif
#include <sys/stat.h>

#ifdef ibmpc
extern char  far *malloc();
#define random() rand()
#define srandom(x) srand(x)
#else
extern char  *malloc();
#endif

#ifndef	EXP_LOCATION
#error	You must define EXP_LOCATION on the command line
#endif

float	*exp_array;	/* table look-up for exp function */
float	exp_mult;
long	exp_add;

extern	int	nn;		/* number of nodes */
extern	int	ni;		/* number of inputs */
extern	int	no;		/* number of outputs */
extern	int	nt;		/* nn + ni + 1 */

extern	int	ce;		/* cross-entropy flag */

extern	int	*outputs;	/* (no) indices of output nodes */
extern	int	*selects;	/* (nn+1) nodes selected for probe printout */

extern	char	root[128];	/* root filename for .cf, .data, .teach, etc.*/

extern	long	sweep;		/* current sweep */
extern	long	err_sweeps;	/* number of sweeps for cumulative error */
extern	long	report;		/* report error every report sweeps */
extern	float	avq_err;	/* error for avq modification */

extern	float	criterion;	/* exit program when rms error < criterion */


float rans(w)
	float	w;
{
/*	extern	long random(); */
#ifdef ibmpc
	static	float max = 32767.0;
#else
	static	float max = 2147483647.0;
#endif
	
	return (((float)random() / max) * 2*w) - w;
}

exp_init()
{
	struct	stat statb;
	int	fd;

	fd = open(EXP_LOCATION, O_RDONLY, 0);

	if (fd < 0) {
		perror("exp_table");
		exit(1);
	}
	fstat(fd, &statb);
	exp_add = (statb.st_size / sizeof(float)) / 2;
	exp_mult = (float) (exp_add / 16);
	exp_array = (float *) malloc(statb.st_size);
	if (read(fd, exp_array, statb.st_size) != statb.st_size) {
		perror("read exp array");
		exit(1);
	}
}

print_nodes(aold)
	float	*aold;
{
	int	i;

	for (i = 1; i <= nn; i++){
		if (selects[i])
			fprintf(stdout,"%7.3f\t",aold[ni+i]);
	}
	fprintf(stdout,"\n");

}

print_output(aold)
	float	*aold;
{
	int	i;

	for (i = 0; i < no; i++){
		fprintf(stdout,"%7.3f\t",aold[ni+outputs[i]]);
	}
	fprintf(stdout,"\n");

}

print_error(e)
	float	*e;
{
	static	int	start = 1;
	static	FILE	*fp;

	FILE	*fopen();
	char	file[128];

	if (start){
		start = 0;
		sprintf(file, "%s.err", root);
		fp = fopen(file, "w");
		if (fp == NULL) {
			perror("ERROR: Can't open .err file");
			exit(1);
		}
	}

	if (ce != 2) {
		/* report rms error */
		*e = sqrt(*e / report);
	} else if (ce == 2) {
		/* report cross-entropy */
		*e = *e / report;
	}
	fprintf(fp,"%g\n",*e);
	fflush(fp);
	if (ce == 0) {
		if (*e < criterion){
			sweep += 1;
			save_wts();
				exit(0);
		}
	}
	*e = 0.;
	err_sweeps = 0;
}

reset_network(aold,anew,apold,apnew)
	float	*aold;
	float	*anew;
	float	***apold;
	float	***apnew;
{
	register	int	i, j, k;

	register	float	*pn;
	register	float	*po;
	register	float	**pnp;
	register	float	**pop;
	register	float	***pnpp;
	register	float	***popp;

	register	float	*zn;
	register	float	*zo;

	zn = anew + 1;
	zo = aold + 1;
	for (i = 1; i < nt; i++, zn++, zo++)
		*zn = *zo = 0.;

	popp = apold;
	pnpp = apnew;
	for (i = 0; i < nn; i++, popp++, pnpp++){
		pop = *popp;
		pnp = *pnpp;
		for (j = 0; j < nt; j++, pop++, pnp++){
			po = *pop;
			pn = *pnp;
			for (k = 0; k < nn; k++, po++, pn++){
				*po = 0.;
				*pn = 0.;
			}
		}
	}

	return;
}

reset_bp_net(aold,anew)
	float	*aold;
	float	*anew;
{
	register	int	i;

	register	float	*zn;
	register	float	*zo;

	zn = anew + 1;
	zo = aold + 1;
	for (i = 1; i < nt; i++, zn++, zo++)
		*zn = *zo = 0.;

	return;
}
