/* Vector read: utilities for reading and writing a vector
 *
 * Author: Dylan Muir (dr.muir@student.qut.edu.au)
 * Date: 14th September, 1999
 * Modified: 16th September, 1999
 * Version: 0.02
 */

#ifndef	__vector_read_h
#define	__vector_read_h

/* -- required modules -- */

#include "vector_utils.h"
#include "stddefs.h"
#include <stdio.h>


/* -- vector reading functions -- */

/* -- Function VectorReadLine
 * Pre: 'input' is a valid file opened for reading
 *      'inFile' is an allocated bool
 * Post: (a vector was read until EOL and was allocated and returned) ||
 *       (there was a problem with allocation && NULL was returned)
 *       'inFile' == (we have not reached EOF)
 */
vector *VectorReadLine(FILE *input, bool *inFile);

/* -- Function VectorReadComponents
 * Pre: 'input' is a valid file openend for reading
 *      'vect' is a valid allocated vector
 *      'inFile' is an allocated bool
 * Post: an entire line of 'input' was read; only the number
 *       of components in 'vect' were assigned; the rest of
 *       the line was discarded
 *       'inFile' == (we have not reached EOF)
 */
void VectorReadComponents(FILE *input, vector *vect, bool *inFile);

/* -- Function VectorWrite
 * Pre: 'output' is a file opened for writing
 *      'vect' is a valid allocated vector
 * Post: 'vect' has been written to 'output'
 * Note: 'vect's components are separated by tabs (\t)
 */
void VectorWrite(FILE *output, vector *vect);

#endif	/* __vector_read_h */

/* --- END of vector_read.h --- */

