#include <stdio.h>
#include "vector_utils.h"
#include "avq_subs.h"
#include "cluster.h"

extern	int	nn;	/* number of nodes */
extern	int	ni;	/* number of inputs */
extern	int	nt;	/* nn + ni + 1 */

extern	long	tsweeps;	/* total sweeps */
extern	long	sweep;		/* current sweep */

extern	float	**wt;		/* (nn x nt): weights */

extern	char	loadfile[];	/* state file base to start with */
extern	char	root[];		/* root file name */

extern	int		avq_flag;	/* flag to perform avq */
extern	vector**	avq_codebook;	/* current AVQ codebook */
extern	int		codebook_size;

save_wts()
{

	FILE	*fp;
	FILE	*fopen();

	register	int	i;
	register	int	j;

	float	*w;
	float	**wp;

	char	weights_file[128],
		means_file[128];

#ifdef ibmpc
	/* 
	 * if running under DOS, probably can't have filenames
	 * with more than 8 chars total, or multiple "."s, so
	 * "fileroot.nnnnn.wts" becomes "fileroot_nnnnn.wts"
	 */
	sprintf(weights_file, "%s_%ld.wts", root, sweep);
	sprintf(means_file, "%s_%ld.mns", root, sweep);
#else
	sprintf(weights_file, "%s.%ld.wts", root, sweep);
	sprintf(means_file, "%s.%ld.means", root, sweep);
#endif
	if ((fp=fopen(weights_file, "w+")) == NULL) {
		perror("Can't open .wts file\n");
		exit(1);
	}
	fprintf(fp, "NETWORK CONFIGURED BY TLEARN\n");
	fprintf(fp, "# weights after %ld sweeps\n", sweep);
	fprintf(fp, "# WEIGHTS\n");

	/* to each node */
	wp = wt;
	for (i = 0; i < nn; i++, wp++){
		fprintf(fp, "# TO NODE %d\n",i+1);
		w = *wp;
		/* from each bias, input, and node */
		for (j = 0; j < nt; j++,w++){
			fprintf(fp,"%f\n",*w);
		}
	}

	fflush(fp);
	fclose(fp);

	if (avq_flag) {
		if (!(fp = fopen(means_file, "w+"))) {
			perror(means_file);
			exit(1);
		}

		WriteCodebook(fp, avq_codebook, codebook_size);

		fflush(fp);
		fclose(fp);
	}

	return;
}

load_wts()
{

	FILE	*fp;
	FILE	*fopen();

	register	int	i;
	register	int	j;

	register	float	*w;
	register	float	**wp;

	int	tmp;

	char	mode[10],
		weights_file[128],
		means_file[128];

	cluster	*saved_book;

	sprintf(weights_file, "%s.wts", loadfile);
	sprintf(means_file, "%s.means", loadfile);

	if ((fp=fopen(weights_file, "r")) == NULL) {
		perror(weights_file);
		exit(1);
	}
	fscanf(fp, "NETWORK CONFIGURED BY %s\n", mode);
	if (strcmp(mode, "TLEARN") != 0) {
		printf("Saved weights not for tlearn-configured network\n");
		exit(1);
	}
	fscanf(fp, "# weights after %ld sweeps\n", &tsweeps);
	fscanf(fp, "# WEIGHTS\n");

	/* to each of nn nodes */
	wp = wt;
	for (i = 0; i < nn; i++, wp++){
		fscanf(fp, "# TO NODE %d\n",&tmp);
		w = *wp;
		/* from each bias, input, and node */
		for (j = 0; j < nt; j++, w++){
			fscanf(fp,"%f\n",w);
		}
	}

	fclose(fp);

	if (avq_flag) {
		if (!(fp = fopen(means_file, "r"))) {
			perror(means_file);
			exit(1);
		}

		saved_book = ClusterReadFromFile(fp);
		avq_codebook = CodebookNewFromCluster(saved_book, &codebook_size);

		fclose(fp);
		saved_book = ClusterDestroy(saved_book);
	}

	return;
}

